#!/usr/bin/env python
# watch.py - script to watch repositories
# Copyright (C) 2010 Joshua D Bartlett
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 3 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

'''
Watches a list of repos, and every minute runs "hg upsync", every 30 minutes
runs "hg synchronise".
'''
import os
import subprocess
import time

SHORT_WAIT = 60
LONG_WAIT = 30*60
SHORT_PER_LONG = LONG_WAIT // SHORT_WAIT
CONFIG_PATH = os.path.expanduser('~/.hgrepos')

def main():
    n = 0
    try:
        while True:
            if n == 0:
                syncRepos(cmd='synchronise')
            else:
                syncRepos(cmd='upsync')
            time.sleep(SHORT_WAIT)
            n = (n + 1) % SHORT_PER_LONG
    except KeyboardInterrupt:
        print '*** Exiting, please wait ***'
    try:
        syncRepos(cmd='synchronise')
    except KeyboardInterrupt:
        print 'My, we are impatient today.'

def syncRepos(cmd):
    print 'Performing hg %s on repos...' % (cmd,)
    for repo in getInterestingRepos():
        p = subprocess.Popen(['hg', cmd, '--repository', repo])
        p.communicate()

def getInterestingRepos():
    if not os.path.isfile(CONFIG_PATH):
        print 'Warning: %r not found' % (CONFIG_PATH,)
        return
    emptyWarning = True
    for line in open(CONFIG_PATH, 'r'):
        line = line.strip()
        if line == '':
            continue
        line = os.path.expanduser(line)
        if os.path.isdir(line):
            print '*** Repository:', line
            yield line
            emptyWarning = False
        else:
            print 'Warning: repository %r not found' % (line,)
            emptyWarning = False
    if emptyWarning:
        print 'Warning: no repositories listed'

if __name__ == '__main__':
    main()
